@@ -56,6 +56,7 @@ from samba.dsdb import _dsdb_load_udv_v2
 from samba.ndr import ndr_pack
 from samba.credentials import SMB_SIGNING_REQUIRED
 from samba import safe_tarfile as tarfile
+import hashlib
 
 
 # work out a SID (based on a free RID) to use when the domain gets restored.
@@ -133,6 +134,14 @@ def backup_filepath(targetdir, name, time_str):
     return os.path.join(targetdir, filename)
 
 
+def create_sha256sum(filename):
+    hash = hashlib.new('sha256')
+    with open(filename, "rb") as f:
+        for chunk in iter(lambda: f.read(65536), b""):
+            hash.update(chunk)
+    return hash.hexdigest()
+
+
 def create_backup_tar(logger, tmpdir, backup_filepath):
     # Adds everything in the tmpdir into a new tar file
     logger.info("Creating backup file %s..." % backup_filepath)
@@ -237,10 +246,13 @@ class cmd_domain_backup_online(samba.netcmd.Command):
                choices=["tdb", "mdb"],
                help="Specify the database backend to be used "
                "(default is %s)" % get_default_backend_store()),
+        Option("--no-sysvol", action="store_true", default=False,
+              help="Exclude sysvol from the backup created"),
+
     ]
 
     def run(self, sambaopts=None, credopts=None, server=None, targetdir=None,
-            no_secrets=False, backend_store=None):
+            no_secrets=False, backend_store=None, no_sysvol=False):
         logger = self.get_logger()
         logger.setLevel(logging.DEBUG)
 
@@ -273,15 +285,16 @@ class cmd_domain_backup_online(samba.netcmd.Command):
             new_sid = get_sid_for_restore(remote_sam, logger)
             realm = remote_sam.domain_dns_name()
 
-            # Grab the remote DC's sysvol files and bundle them into a tar file
-            logger.info("Backing up sysvol files (via SMB)...")
-            sysvol_tar = os.path.join(tmpdir, 'sysvol.tar.gz')
-            smb_conn = smb_sysvol_conn(server, lp, creds)
-            backup_online(smb_conn, sysvol_tar, remote_sam.get_domain_sid())
+            if not no_sysvol:
+                # Grab the remote DC's sysvol files and bundle them into a tar file
+                logger.info("Backing up sysvol files (via SMB)...")
+                sysvol_tar = os.path.join(tmpdir, 'sysvol.tar.gz')
+                smb_conn = smb_sysvol_conn(server, lp, creds)
+                backup_online(smb_conn, sysvol_tar, remote_sam.get_domain_sid())
 
-            # remove the default sysvol files created by the clone (we want to
-            # make sure we restore the sysvol.tar.gz files instead)
-            shutil.rmtree(paths.sysvol)
+                # remove the default sysvol files created by the clone (we want to
+                # make sure we restore the sysvol.tar.gz files instead)
+                shutil.rmtree(paths.sysvol)
 
             # Edit the downloaded sam.ldb to mark it as a backup
             samdb = SamDB(url=paths.samdb, session_info=system_session(), lp=lp,
@@ -491,6 +504,8 @@ class cmd_domain_backup_restore(cmd_fsmo_seize):
         tf = tarfile.open(backup_file)
         tf.extractall(targetdir)
         tf.close()
+        # set the permissions on the target dir strict but also relaxed enough
+        os.chmod(targetdir, mode=0o755)
 
         # use the smb.conf that got backed up, by default (save what was
         # actually backed up, before we mess with it)
@@ -676,8 +691,11 @@ class cmd_domain_backup_restore(cmd_fsmo_seize):
         dest_sysvol_dir = lp.get('path', 'sysvol')
         if not os.path.exists(dest_sysvol_dir):
             os.makedirs(dest_sysvol_dir)
-        backup_restore(sysvol_tar, dest_sysvol_dir, samdb, smbconf)
-        os.remove(sysvol_tar)
+        if os.path.isfile(sysvol_tar):
+            backup_restore(sysvol_tar, dest_sysvol_dir, samdb, smbconf)
+            os.remove(sysvol_tar)
+        else:
+            logger.warning("back-up has no sysvol data")
 
         # fix up any stale links to the old DCs we just removed
         logger.info("Fixing up any remaining references to the old DCs...")
@@ -1172,7 +1190,8 @@ class cmd_domain_backup_offline(samba.netcmd.Command):
         # Close the original samdb, to avoid any confusion, we will
         # not use this any more as the data has all been copied under
         # the transaction
-        samdb = None
+        samdb.disconnect()
+        del samdb
 
         # Open the new backed up samdb, flag it as backed up, and write
         # the next SID so the restore tool can add objects. We use
@@ -1189,7 +1208,8 @@ class cmd_domain_backup_offline(samba.netcmd.Command):
         add_backup_marker(samdb, "backupType", "offline")
 
         # Close the backed up samdb
-        samdb = None
+        samdb.disconnect()
+        del samdb
 
         # Now handle all the LDB and TDB files that are not linked to
         # anything else.  Use transactions for LDBs.
@@ -1226,20 +1246,36 @@ class cmd_domain_backup_offline(samba.netcmd.Command):
         os.remove(backup_fn)
 
         logger.info('building backup tar')
+
+        chksum_list = []
+
         for path in all_files:
             arc_path = self.get_arc_path(path, paths)
 
             if os.path.exists(path + self.backup_ext):
                 logger.info('   adding backup ' + arc_path + self.backup_ext +
                             ' to tar and deleting file')
+                chksum_list.append(
+                    "%s  %s" % (create_sha256sum(path + self.backup_ext),
+                                arc_path))
                 tar.add(path + self.backup_ext, arcname=arc_path)
                 os.remove(path + self.backup_ext)
             elif path.endswith('.ldb') or path.endswith('.tdb'):
                 logger.info('   skipping ' + arc_path)
-            else:
+            elif os.path.isfile(path):
                 logger.info('   adding misc file ' + arc_path)
+                chksum_list.append("%s  %s" %
+                                   (create_sha256sum(path),
+                                    arc_path))
                 tar.add(path, arcname=arc_path)
 
+        chksum_filepath = os.path.join(temp_tar_dir, "SHA256SUM")
+        with open(chksum_filepath, "w") as f:
+            for c in chksum_list:
+                f.write(c + '\n')
+        tar.add(chksum_filepath, os.path.basename(chksum_filepath))
+        os.remove(chksum_filepath)
+
         tar.close()
         os.rename(temp_tar_name,
                   os.path.join(targetdir,
